uniform sampler2D tex;
uniform float Exaggeration;
varying vec3 normal,lightDir;
varying vec4 vpos;
varying vec3 nxtan, nytan;

void main()
{
  vec2 st = vec2(1.0 - gl_TexCoord[0].s, gl_TexCoord[0].t);  // seemingly necessary to get texture in correct orientation

  // figure out x and y bump displacement from height map

  // using known dimensions of height map here to figure out proper (ds, dt) to 
  // move over one texel horiztonally and vertically

  float ds = 1.0 / 1024.0;
  float dt = 1.0 / 512.0;

  float height = texture2D(tex, st).r;
  float height_ds = texture2D(tex, st + vec2(ds, 0)).r;
  float height_dt = texture2D(tex, st + vec2(0, dt)).r;
  
  // now we have horiztonal slope, vertical slope

  float dheight_ds = height_ds - height;
  float dheight_dt = height_dt - height;

  // diffuse lighting stuff

  vec3 n = normalize(normal);
  vec3 l = normalize(lightDir);

  // here is where we calculate the perturbed normal by scaling our tangent and bitangent vectors
  // by the slopes calculated above and adding them to the original normal

  float NdotL;

  vec3 nxt = normalize(nxtan);
  vec3 nyt = normalize(nytan);

  vec3 nprime = normalize(n + Exaggeration * (dheight_ds * nxt + dheight_dt * nyt));
  NdotL = dot(nprime, l);

  // final color is standard diffuse formulation

  vec4 color;

  color = vec4(vec3(1.0, 1.0, 1.0) * NdotL, 1.0);

  gl_FragColor = color;
}
